SystemOrganization addCategory: #'ICU-Core'!

ExternalStructure subclass: #UErrorCode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!UErrorCode classSide methodsFor: 'field definition' stamp: 'jvds 6/11/2011 22:54'!
fields

	^#(
		(status 'long')
	)! !

!UErrorCode methodsFor: 'testing' stamp: 'jvds 6/12/2011 13:06'!
failure

	^self status > 0! !

!UErrorCode methodsFor: 'accessing' stamp: 'jvds 6/11/2011 22:55'!
status

	^handle signedLongAt: 1! !

!UErrorCode methodsFor: 'testing' stamp: 'jvds 6/12/2011 12:59'!
success

	^self status <= 0! !

SharedPool subclass: #ICUFunctions
	instanceVariableNames: ''
	classVariableNames: 'INModule UCModule UCalGetDefaultTimeZone UCalOpenTimeZones UEnumClose UEnumCount UEnumNext UEnumReset UEnumUNext VersionSuffix'
	poolDictionaries: ''
	category: 'ICU-Core'!

!ICUFunctions commentStamp: 'jvds 6/13/2011 17:00' prior: 0!
ICUFunctions is a pool dictionary with all required ICU functions. The functions are not declared as Smalltalk api methods because they look a little bit different on each platform.

The ICU - International Components for Unicode Library (icu4c) can be downloaded from: http://site.icu-project.org/

Win32
	Add the ICU bin directory to the PATH environment variable.
	Note that the ICU libraries on Windows require the Microsoft Visual C++ 2010 Redistributable Package

MaxOSX
	Get the 32bit versions of the ICU libraries
	For example by using MacPorts (on Snow Leapard set the build_arch configuration property to i386 to get 32bit libraries).
	Create a symbolic link to the libraries from inside the Resources directory of the VM. For example
		ln -s /opt/local/lib/libicuuc.dylib .
		ln -s /opt/local/lib/libicui18n .
	
Unix
	coming soon ...!

!ICUFunctions classSide methodsFor: 'internal' stamp: 'jvds 6/13/2011 11:48'!
callType

	^ExternalFunction callTypeCDecl 	! !

!ICUFunctions classSide methodsFor: 'initialization' stamp: 'jvds 6/13/2011 14:16'!
initialize 

	self initializeModulesAndVersions.
	self initializeEnumFunctions.
	self initializeCalendarFunctions.! !

!ICUFunctions classSide methodsFor: 'initialization' stamp: 'jvds 6/13/2011 14:45'!
initializeCalendarFunctions

	UCalGetDefaultTimeZone := self newINFunction: 'ucal_getDefaultTimeZone'
		returnType: self types long
		argumentTypes: {
			self types void asPointerType.
			self types long.
			UErrorCode  externalType asPointerType }.

	UCalOpenTimeZones := self newINFunction: 'ucal_openTimeZones' 
		returnType: UEnumerator externalType asPointerType 
		argumentTypes: { UErrorCode  externalType asPointerType }! !

!ICUFunctions classSide methodsFor: 'initialization' stamp: 'jvds 6/13/2011 14:16'!
initializeEnumFunctions

	"const char* uenum_next (UEnumeration *en, int32_t * resultLength, UErrorCode * 	status )	"
	UEnumNext := self newUCFunction: 'uenum_next' 
		returnType: self types char asPointerType 
		argumentTypes: {
			UEnumerator externalType asPointerType.
			self types long asPointerType.
			UErrorCode  externalType asPointerType }.
	
	"void uenum_close (UEnumeration * 	en)"
	UEnumClose := self newUCFunction: 'uenum_close'
		returnType: self types void
		argumentTypes: { UEnumerator externalType asPointerType }.! !

!ICUFunctions classSide methodsFor: 'initialization' stamp: 'jvds 6/13/2011 16:43'!
initializeModulesAndVersions
	"Common Library	icuucXY.dll	libicuuc.so.XY.Z	Base library required by all other ICU libraries."
	"Internationalization (i18n) Library	icuinXY.dll	libicui18n.so.XY.Z	A library that contains many locale based internationalization (i18n) functions."
	
	UCModule := 'libicuuc'.
	INModule := 'libicui18n'.
	VersionSuffix := '_48'.

	Smalltalk os isWin32
		ifTrue: [
			UCModule := 'icuuc48.dll'.
			INModule := 'icuin48.dll'.
			VersionSuffix := '_48' ].
	Smalltalk os isMacOS
		ifTrue: [
			UCModule := 'libicuuc.dylib'.
			INModule := 'libicui18n.dylib'.
			VersionSuffix := '_46' ].! !

!ICUFunctions classSide methodsFor: 'internal' stamp: 'jvds 6/13/2011 14:15'!
newINFunction: aName returnType: aReturnType argumentTypes: anArgumentTypes

	^ExternalLibraryFunction name: (aName, VersionSuffix)
		module: INModule
		callType: self callType
		returnType: aReturnType
		argumentTypes: anArgumentTypes ! !

!ICUFunctions classSide methodsFor: 'internal' stamp: 'jvds 6/13/2011 14:15'!
newUCFunction: aName returnType: aReturnType argumentTypes: anArgumentTypes

	^ExternalLibraryFunction name: (aName, VersionSuffix)
		module: UCModule
		callType: self callType
		returnType: aReturnType
		argumentTypes: anArgumentTypes ! !

!ICUFunctions classSide methodsFor: 'internal' stamp: 'jvds 6/13/2011 11:43'!
types

	^ExternalType ! !

ExternalStructure subclass: #ICUGeneralObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'ICUFunctions'
	category: 'ICU-Core'!

!ICUGeneralObject classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/12/2011 13:06'!
checkUErrorCode: anUErrorCode valueRC: rc

	"some kind of exception throwing should be here ..."
	anUErrorCode  failure ifTrue:[ self halt ].
	^rc value! !

!ICUGeneralObject classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/11/2011 23:14'!
fields

	^#(
		(nil 'ulong')
	)! !

!ICUGeneralObject classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/13/2011 15:09'!
stringFromUTF16: aBuffer

	^(GRCodec forEncoding: 'utf-16') decode: aBuffer! !

!ICUGeneralObject classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/13/2011 15:05'!
stringFromUTF16: aBuffer size: aSize

	^(GRCodec forEncoding: 'utf-16') decode: (aBuffer copyFrom: 1 to: aSize)! !

ICUGeneralObject subclass: #UCalendar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!UCalendar classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/13/2011 14:17'!
allTimeZones

	| enumerator status list |
		
	enumerator := UCalOpenTimeZones invokeWith: (status := UErrorCode new).
	list := enumerator collect: [ :each | each ].
	enumerator close.
	^list! !

!UCalendar classSide methodsFor: 'as yet unclassified' stamp: 'jvds 6/13/2011 15:09'!
getDefaultTimeZone

	| buffer bufferSize actualSize status |
	
	bufferSize := 80.
	buffer := ByteArray new: bufferSize * 2.
	
	actualSize := UCalGetDefaultTimeZone invokeWith: buffer with: bufferSize with: (status := UErrorCode new).

	^self stringFromUTF16: (buffer copyFrom: 1 to: actualSize * 2)! !

ICUGeneralObject subclass: #UEnumerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ICU-Core'!

!UEnumerator methodsFor: 'actions' stamp: 'jvds 6/13/2011 13:56'!
close

	UEnumClose invokeWith: self ! !

!UEnumerator methodsFor: 'enumerating' stamp: 'jvds 6/12/2011 00:06'!
collect: aBlock

	| nextItem result |
	
	result := OrderedCollection new.
	nextItem := self next.
	[ nextItem isNil ] whileFalse:[
		result add: (aBlock value: nextItem).
		nextItem := self next.
	].
	^result! !

!UEnumerator methodsFor: 'accessing' stamp: 'jvds 6/13/2011 14:18'!
next

	| result length status |
	
	result := UEnumNext invokeWith: self with: (length := WordArray new: 1) with: (status := UErrorCode new).
	^self class checkUErrorCode: status valueRC: result! !
ICUFunctions initialize!
